import json
from dimorfism.setup_data import split_data
from dimorfism.fit_logistic_regression import get_fitted_model


def get_model_parameters(selected_data):
    splited_data = split_data(selected_data)
    fitted_model = get_fitted_model(splited_data)
    parameters_dictionary = obtained_parameters(fitted_model)
    return parameters_dictionary


def obtained_parameters(fitted_model):
    keys = [str(name) for name in fitted_model.feature_names_in_] + ["Intercept"]
    values = [*fitted_model.coef_[0], fitted_model.intercept_.item()]
    return {k: float(v) for (k, v) in zip(keys, values)}


def write_json_parameters(parameters_dictionary, parameters_path):
    with open(parameters_path, "w") as outfile:
        json.dump(parameters_dictionary, outfile)



import pandas as pd


def adapter_morphometry(data_path):
    wanted_colnames: list = [
        "bill_depth",
        "bill_length",
        "head_width",
        "Tarsus",
    ]
    complete_dataframe = pd.read_csv(data_path)
    return complete_dataframe[wanted_colnames + ["sexo"]]



from sklearn import linear_model


def get_fitted_model(splited_data):
    model = logistic_regression()
    x = splited_data["to_fit"]
    y = splited_data["to_fit_target"]
    fitted_model = model.fit(x, y)
    return fitted_model


def logistic_regression():
    return linear_model.LogisticRegression()



import pandas as pd
from sklearn.model_selection import train_test_split


def split_data(df: pd.DataFrame) -> dict:
    target_name = "sexo"
    target = df[target_name]
    data_train, data_test, target_train, target_test = train_test_split(
        df, target, random_state=7, train_size=0.8
    )
    splitted_data = {
        "to_fit": data_train.drop(columns=target_name),
        "to_fit_target": target_train,
        "to_test": data_test.drop(columns=target_name),
        "to_test_y": pd.DataFrame(target_test),
    }

    return splitted_data



from dimorfism.write_parameters import get_model_parameters, write_json_parameters
from dimorfism.adapters import adapter_morphometry

import typer

app = typer.Typer()


@app.command()
def write_model_parameters(
    data_path: str = typer.Option("Input morphometry data path"),
    parameters_path: str = typer.Option("Output parameters path"),
):
    selected_dataframe = adapter_morphometry(data_path)
    parameters_dictionary = get_model_parameters(selected_dataframe)
    write_json_parameters(parameters_dictionary, parameters_path)


@app.command()
def version():
    print("0.1.0")
